@@ -17,15 +17,14 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include <cctype>
 #include <iostream>
-#include <iomanip>
 
-#include <osmscout/Database.h>
-#include <osmscout/LocationService.h>
-#include <osmscout/LocationDescriptionService.h>
+#include <osmscout/db/Database.h>
 
-#include <osmscout/util/CmdLineParsing.h>
+#include <osmscout/location/LocationService.h>
+#include <osmscout/location/LocationDescriptionService.h>
+
+#include <osmscout/cli/CmdLineParsing.h>
 
 struct Arguments
 {
@@ -379,7 +378,7 @@ int main(int argc, char* argv[])
                         args.repeat=value;
                       }),
                       "repeat",
-                      "Cout of repeat for performance test");
+                      "Count of repeat for performance test");
 
   argParser.AddOption(osmscout::CmdLineFlag([&args](const bool& value) {
                         args.transliterate=value;
@@ -392,7 +391,7 @@ int main(int argc, char* argv[])
                             args.databaseDirectory=value;
                           }),
                           "DATABASE",
-                          "Directory of the database to use");
+                          "Directory of the db to use");
 
   argParser.AddPositional(osmscout::CmdLineStringListOption([&args](const std::string& value) {
                             args.location.push_back(value);
@@ -436,10 +435,10 @@ int main(int argc, char* argv[])
   }
 
   osmscout::DatabaseParameter databaseParameter;
-  osmscout::DatabaseRef       database(new osmscout::Database(databaseParameter));
+  osmscout::DatabaseRef       database=std::make_shared<osmscout::Database>(databaseParameter);
 
   if (!database->Open(args.databaseDirectory)) {
-    std::cerr << "Cannot open database" << std::endl;
+    std::cerr << "Cannot open db" << std::endl;
 
     return 1;
   }
@@ -500,6 +499,9 @@ int main(int argc, char* argv[])
 
   std::cout << "Database:                " << args.databaseDirectory << std::endl;
   std::cout << "Search pattern:          " << searchParameter.GetSearchString() << std::endl;
+  if (args.transliterate) {
+    std::cout << "Transliterated pattern:  " << osmscout::UTF8Transliterate(osmscout::UTF8StringToUpper(searchParameter.GetSearchString())) << std::endl;
+  }
   std::cout << "Search for location:     " << (searchParameter.GetSearchForLocation() ? "true" : "false") << std::endl;
   std::cout << "Search for POI:          " << (searchParameter.GetSearchForPOI() ? "true" : "false") << std::endl;
   std::cout << "Admin region only match: " << (searchParameter.GetAdminRegionOnlyMatch() ? "true" : "false") << std::endl;
