@@ -31,9 +31,10 @@
 #include <iostream>
 #include <fstream>
 #include <cmath>
-#include <cstring>
+#include <optional>
+
+#include <osmscout/db/Database.h>
 
-#include <osmscout/Database.h>
 #include <osmscoutmap/MapService.h>
 
 #include <osmscout/routing/SimpleRoutingService.h>
@@ -48,18 +49,19 @@
 #include <osmscout/navigation/ArrivalEstimateAgent.h>
 #include <osmscout/navigation/SpeedAgent.h>
 
-#include <osmscout/util/CmdLineParsing.h>
+#include <osmscout/cli/CmdLineParsing.h>
 
 struct Arguments
 {
-  bool                   help=false;
-  bool                   debug=false;
-  std::string            router=osmscout::RoutingService::DEFAULT_FILENAME_BASE;
-  osmscout::Vehicle      vehicle=osmscout::Vehicle::vehicleCar;
-  std::string            databaseDirectory;
-  osmscout::GeoCoord     start;
-  osmscout::GeoCoord     target;
-  std::string            gpxFile;
+  bool                              help=false;
+  bool                              debug=false;
+  std::string                       router=osmscout::RoutingService::DEFAULT_FILENAME_BASE;
+  osmscout::Vehicle                 vehicle=osmscout::Vehicle::vehicleCar;
+  std::string                       databaseDirectory;
+  osmscout::GeoCoord                start;
+  osmscout::GeoCoord                target;
+  std::optional<osmscout::Bearing>  initialBearing;
+  std::string                       gpxFile;
 };
 
 class ConsoleRoutingProgress : public osmscout::RoutingProgress
@@ -162,7 +164,6 @@ PathGenerator::PathGenerator(const osmscout::RouteDescription& description,
                              double maxSpeed)
 {
   size_t             tickCount=0;
-  double             totalTime=0.0;
   double             restTime=0.0;
   auto               currentNode=description.Nodes().begin();
   auto               nextNode=currentNode;
@@ -204,7 +205,6 @@ PathGenerator::PathGenerator(const osmscout::RouteDescription& description,
     auto timeInHours=distanceInKilometer/maxSpeed;
     auto timeInSeconds=timeInHours*60*60;
 
-    totalTime+=timeInHours;
 
     // Make sure we do not skip edges in the street
     lastPosition=currentNode->GetLocation();
@@ -560,11 +560,17 @@ int main(int argc, char* argv[])
                       "gpxFile",
                       "Name of the gpx file containing the route");
 
+  argParser.AddOption(osmscout::CmdLineDoubleOption([&args](double value) {
+                        args.initialBearing=osmscout::Bearing::Degrees(value);
+                      }),
+                      "initial-bearing",
+                      "Initial vehicle bearing (degrees, North is 0, East is 90...).");
+
   argParser.AddPositional(osmscout::CmdLineStringOption([&args](const std::string& value) {
                             args.databaseDirectory=value;
                           }),
                           "DATABASE",
-                          "Directory of the first database to use");
+                          "Directory of the first db to use");
 
   argParser.AddPositional(osmscout::CmdLineGeoCoordOption([&args](const osmscout::GeoCoord& value) {
                             args.start=value;
@@ -597,7 +603,7 @@ int main(int argc, char* argv[])
   osmscout::DatabaseRef       database=std::make_shared<osmscout::Database>(databaseParameter);
 
   if (!database->Open(args.databaseDirectory)) {
-    std::cerr << "Cannot open database" << std::endl;
+    std::cerr << "Cannot open db" << std::endl;
 
     return 1;
   }
@@ -610,7 +616,7 @@ int main(int argc, char* argv[])
                                                                                             args.router);
 
   if (!router->Open()) {
-    std::cerr << "Cannot open routing database" << std::endl;
+    std::cerr << "Cannot open routing db" << std::endl;
 
     return 1;
   }
@@ -669,6 +675,7 @@ int main(int argc, char* argv[])
   auto routingResult=router->CalculateRoute(*routingProfile,
                                             start,
                                             target,
+                                            args.initialBearing,
                                             parameter);
 
   if (!routingResult.Success()) {
