@@ -23,23 +23,23 @@
 #include <QDebug>
 
 #include <chrono>
-#include <cstring>
 #include <iostream>
 #include <list>
+#include <optional>
 
-#include <osmscout/Database.h>
-#include <osmscoutmap/MapService.h>
-
-#include <osmscout/util/Projection.h>
-#include <osmscout/util/Geometry.h>
-#include <osmscout/util/CmdLineParsing.h>
-
-#include <osmscoutmapqt/MapPainterQt.h>
+#include <osmscout/projection/MercatorProjection.h>
 
 #include <osmscout/routing/SimpleRoutingService.h>
-#include <osmscout/routing/RoutePostprocessor.h>
 #include <osmscout/routing/RoutingService.h>
 
+#include <osmscout/db/Database.h>
+
+#include <osmscout/util/Geometry.h>
+#include <osmscout/cli/CmdLineParsing.h>
+
+#include <osmscoutmap/MapService.h>
+
+#include <osmscoutmapqt/MapPainterQt.h>
 
 //#define ROUTE_DEBUG
 //#define NODE_DEBUG
@@ -107,10 +107,11 @@ void drawDot(QPainter &painter,
              const osmscout::MercatorProjection &projection,
              const osmscout::GeoCoord &coord){
 
-  double x,y;
-  projection.GeoToPixel(coord,x,y);
+  osmscout::Vertex2D pos;
+  projection.GeoToPixel(coord,
+                        pos);
   painter.setPen(Qt::NoPen);
-  painter.drawEllipse(x,y,10,10);
+  painter.drawEllipse(pos.GetX(),pos.GetY(),10,10);
 }
 
 class RoutingServiceAnimation: public osmscout::SimpleRoutingService{
@@ -148,17 +149,14 @@ public:
   {
   }
 
-  virtual ~RoutingServiceAnimation()
-  {
-  }
+  ~RoutingServiceAnimation() override = default;
 
-  virtual bool WalkToOtherDatabases(const osmscout::RoutingProfile& /*state*/,
-                                    osmscout::RoutingService::RNodeRef &current,
-                                    osmscout::RouteNodeRef &/*currentRouteNode*/,
-                                    osmscout::RoutingService::OpenList &openList,
-                                    osmscout::RoutingService::OpenMap &/*openMap*/,
-                                    const osmscout::RoutingService::ClosedSet &closedSet,
-                                    const ClosedSet &closedRestrictedSet)
+  bool WalkToOtherDatabases(const osmscout::RoutingProfile& /*state*/,
+                            osmscout::RoutingService::RNodeRef &current,
+                            osmscout::RouteNodeRef &/*currentRouteNode*/,
+                            osmscout::RoutingService::OpenList &openList,
+                            osmscout::RoutingService::OpenMap &/*openMap*/,
+                            const osmscout::RoutingService::ClosedSet &closedSet) override
   {
     if (stepCounter<startStep || (endStep>0 && (int64_t)stepCounter>endStep)){
       stepCounter++;
@@ -170,8 +168,6 @@ public:
     }
     stepCounter++;
 
-    double x1,y1;
-    double x2,y2;
     osmscout::RouteNodeRef n1;
     osmscout::RouteNodeRef n2;
 
@@ -193,9 +189,6 @@ public:
     pen.setWidth(5);
 
     // draw ways in closedSet
-    painter.setBrush(QBrush(red));
-    pen.setColor(red);
-
     for (const auto &closedNode:closedSet){
       if (!closedNode.currentNode.IsValid() ||
           !closedNode.previousNode.IsValid()){
@@ -206,30 +199,24 @@ public:
         return false;
       }
 
-      projection.GeoToPixel(n1->GetCoord(),x1,y1);
-      projection.GeoToPixel(n2->GetCoord(),x2,y2);
-      painter.setPen(pen);
-      painter.drawLine(x1,y1,x2,y2);
-    }
-
-    // closed, restricted
-    painter.setBrush(QBrush(grey));
-    pen.setColor(grey);
-
-    for (const auto &closedNode:closedRestrictedSet){
-      if (!closedNode.currentNode.IsValid() ||
-          !closedNode.previousNode.IsValid()){
-        continue;
-      }
-      if (!GetRouteNode(closedNode.currentNode,n1) ||
-          !GetRouteNode(closedNode.previousNode,n2)){
-        return false;
+      if (closedNode.currentRestricted){
+        painter.setBrush(QBrush(grey));
+        pen.setColor(grey);
+      } else {
+        painter.setBrush(QBrush(red));
+        pen.setColor(red);
       }
 
-      projection.GeoToPixel(n1->GetCoord(),x1,y1);
-      projection.GeoToPixel(n2->GetCoord(),x2,y2);
+      osmscout::Vertex2D pos1;
+      osmscout::Vertex2D pos2;
+
+      projection.GeoToPixel(n1->GetCoord(),
+                            pos1);
+      projection.GeoToPixel(n2->GetCoord(),
+                            pos2);
       painter.setPen(pen);
-      painter.drawLine(x1,y1,x2,y2);
+      painter.drawLine(pos1.GetX(),pos1.GetY(),
+                       pos2.GetX(),pos2.GetY());
     }
 
     // draw nodes in open list
@@ -242,10 +229,16 @@ public:
         if (!GetRouteNode(open->prev,n1)){
           return false;
         }
-        projection.GeoToPixel(n1->GetCoord(),x1,y1);
-        projection.GeoToPixel(open->node->GetCoord(),x2,y2);
+        osmscout::Vertex2D pos1;
+        osmscout::Vertex2D pos2;
+
+        projection.GeoToPixel(n1->GetCoord(),
+                              pos1);
+        projection.GeoToPixel(open->node->GetCoord(),
+                              pos2);
         painter.setPen(pen);
-        painter.drawLine(x1,y1,x2,y2);
+        painter.drawLine(pos1.GetX(),pos1.GetY(),
+                         pos2.GetX(),pos2.GetY());
       }
     }
 
@@ -257,10 +250,17 @@ public:
       if (!GetRouteNode(current->prev,n1)){
         return false;
       }
-      projection.GeoToPixel(n1->GetCoord(),x1,y1);
-      projection.GeoToPixel(current->node->GetCoord(),x2,y2);
+
+      osmscout::Vertex2D pos1;
+      osmscout::Vertex2D pos2;
+
+      projection.GeoToPixel(n1->GetCoord(),
+                            pos1);
+      projection.GeoToPixel(current->node->GetCoord(),
+                            pos2);
       painter.setPen(pen);
-      painter.drawLine(x1,y1,x2,y2);
+      painter.drawLine(pos1.GetX(),pos1.GetY(),
+                       pos2.GetX(),pos2.GetY());
     }
 
     painter.end();
@@ -470,7 +470,7 @@ int main(int argc, char* argv[])
                             args.databaseDirectory=value;
                           }),
                           "DATABASE",
-                          "Directory of the database to use");
+                          "Directory of the db to use");
 
   argParser.AddPositional(osmscout::CmdLineGeoCoordOption([&args](const osmscout::GeoCoord& value) {
                             args.start=value;
@@ -507,7 +507,7 @@ int main(int argc, char* argv[])
   osmscout::MapServiceRef     mapService(new osmscout::MapService(database));
 
   if (!database->Open(args.databaseDirectory.c_str())) {
-    std::cerr << "Cannot open database" << std::endl;
+    std::cerr << "Cannot open db" << std::endl;
 
     return 1;
   }
@@ -594,7 +594,7 @@ int main(int argc, char* argv[])
                                                     args.startFrame);
 
   if (!router->Open()) {
-    std::cerr << "Cannot open routing database" << std::endl;
+    std::cerr << "Cannot open routing db" << std::endl;
 
     return 1;
   }
@@ -654,6 +654,7 @@ int main(int argc, char* argv[])
   osmscout::RoutingResult result=router->CalculateRoute(routingProfile,
                                                         start,
                                                         target,
+                                                        std::nullopt,
                                                         parameter);
 
   if (!result.Success()) {
