@@ -27,12 +27,14 @@
 #include <vector>
 #include <optional>
 
-#include <osmscout/util/CmdLineParsing.h>
-#include <osmscout/util/File.h>
+#include <osmscout/cli/CmdLineParsing.h>
+#include <osmscout/util/String.h>
+
+#include <osmscout/io/File.h>
 
 #include <osmscoutimport/Import.h>
 
-static std::string VehcileMaskToString(osmscout::VehicleMask vehicleMask)
+static std::string VehicleMaskToString(osmscout::VehicleMask vehicleMask)
 {
   std::string result;
 
@@ -123,6 +125,9 @@ void DumpHelp(osmscout::ImportParameter& parameter)
   std::cout << " --wayDataMemoryMaped true|false      memory maped way data file access (default: " << osmscout::BoolToString(parameter.GetWayDataMemoryMaped()) << ")" << std::endl;
   std::cout << " --wayDataCacheSize <number>          way data cache size (default: " << parameter.GetWayDataCacheSize() << ")" << std::endl;
 
+  std::cout << " --areaWayIndexMinMag <number>        minimum index level for area way index analysis (default: " << parameter.GetAreaWayIndexMinMag() << ")" << std::endl;
+  std::cout << " --areaWayIndexMaxMag <number>        maximum index level for area way index analysis (default: " << parameter.GetAreaWayIndexMaxMag() << ")" << std::endl;
+
   std::cout << " --routeNodeBlockSize <number>        number of route nodes resolved in block (default: " << parameter.GetRouteNodeBlockSize() << ")" << std::endl;
   std::cout << std::endl;
   std::cout << " --langOrder <#|lang1[,#|lang2]..>    language order when parsing lang[:language] and place_name[:language] tags" << std::endl
@@ -303,7 +308,7 @@ static void DumpParameter(const osmscout::ImportParameter& parameter,
 
 
   for (const auto& router : parameter.GetRouter()) {
-    progress.Info(std::string("Router: ")+VehcileMaskToString(router.GetVehicleMask())+ " - '"+router.GetFilenamebase()+"'");
+    progress.Info(std::string("Router: ") + VehicleMaskToString(router.GetVehicleMask()) + " - '" + router.GetFilenamebase() + "'");
   }
 
   progress.Info(std::string("StrictAreas: ")+
@@ -347,6 +352,10 @@ static void DumpParameter(const osmscout::ImportParameter& parameter,
                 (parameter.GetAreaDataMemoryMaped() ? "true" : "false"));
   progress.Info(std::string("AreaDataCacheSize: ")+
                 std::to_string(parameter.GetAreaDataCacheSize()));
+  progress.Info(std::string("AreaWayIndexMinMag: ")+
+                std::to_string(parameter.GetAreaWayIndexMinMag().Get()));
+  progress.Info(std::string("AreaWayIndexMaxMag: ")+
+                std::to_string(parameter.GetAreaWayIndexMaxMag().Get()));
 
   progress.Info(std::string("WayDataMemoryMaped: ")+
                 (parameter.GetWayDataMemoryMaped() ? "true" : "false"));
@@ -808,6 +817,32 @@ int main(int argc, char* argv[])
         parameterError=true;
       }
     }
+    else if (strcmp(argv[i],"--areaWayIndexMinMag")==0) {
+      size_t areaWayIndexMinMag;
+
+      if (osmscout::ParseSizeTArgument(argc,
+                                       argv,
+                                       i,
+                                       areaWayIndexMinMag)) {
+        parameter.SetAreaWayIndexMinMag(osmscout::MagnificationLevel(areaWayIndexMinMag));
+      }
+      else {
+        parameterError=true;
+      }
+    }
+    else if (strcmp(argv[i],"--areaWayIndexMaxMag")==0) {
+      size_t areaWayIndexMaxMag;
+
+      if (osmscout::ParseSizeTArgument(argc,
+                                       argv,
+                                       i,
+                                       areaWayIndexMaxMag)) {
+        parameter.SetAreaWayIndexMaxMag(osmscout::MagnificationLevel(areaWayIndexMaxMag));
+      }
+      else {
+        parameterError=true;
+      }
+    }
     else if (strcmp(argv[i],"--wayDataMemoryMaped")==0) {
       bool wayDataMemoryMaped;
 
