@@ -18,9 +18,14 @@
  */
 
 #include <osmscoutclientqt/LookupModule.h>
-#include <osmscoutclientqt/OSMScoutQt.h>
+
+
 #include <iostream>
 
+#include <osmscout/feature/AdminLevelFeature.h>
+
+#include <osmscoutclientqt/OSMScoutQt.h>
+
 namespace osmscout {
 
 LookupModule::LookupModule(QThread *thread,DBThreadRef dbThread):
@@ -29,9 +34,7 @@ LookupModule::LookupModule(QThread *thread,DBThreadRef dbThread):
   dbThread(dbThread),
   loadJob(nullptr)
 {
-
-  connect(dbThread.get(), &DBThread::initialisationFinished,
-          this, &LookupModule::initialisationFinished);
+  dbThread->databaseLoadFinished.Connect(dbLoadedSlot);
 }
 
 LookupModule::~LookupModule()
@@ -70,7 +73,7 @@ void LookupModule::requestObjectsOnView(const MapViewStruct &view,
   connect(loadJob, &DBLoadJob::finished,
           this, &LookupModule::onLoadJobFinished);
 
-  dbThread->RunJob(loadJob);
+  dbThread->RunJob(std::bind(&DBLoadJob::Run, loadJob, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
 }
 
 void LookupModule::addObjectInfo(QList<ObjectInfo> &objectList, // output
@@ -144,7 +147,7 @@ void LookupModule::requestObjects(const LocationEntry &entry, bool reverseLookup
   dbThread->RunSynchronousJob(
     [&](const std::list<DBInstanceRef> &databases){
       for (const auto &db:databases) {
-        if (db->path==entry.getDatabase()){
+        if (QString::fromStdString(db->path)==entry.getDatabase()){
 
           std::set<osmscout::FileOffset>         areaOffsets;
           std::set<osmscout::FileOffset>         wayOffsets;
@@ -168,7 +171,7 @@ void LookupModule::requestObjects(const LocationEntry &entry, bool reverseLookup
 
           qDebug() << "Lookup objects for location entry" << entry.getLabel() << ":"
                    << areaOffsets.size() << wayOffsets.size() << nodeOffsets.size()
-                   << "(areas, ways, nodes) in database" << entry.getDatabase();
+                   << "(areas, ways, nodes) in db" << entry.getDatabase();
 
           MapData mapData;
 
@@ -224,15 +227,12 @@ void LookupModule::filterObjectInView(const osmscout::MapData &mapData,
   std::map<osmscout::FileOffset,osmscout::AdminRegionRef> regionMap;
   DBInstanceRef db;
 
-  double x;
-  double y;
-  double x2;
-  double y2;
-
   //std::cout << "nodes: " << d.nodes.size() << std::endl;
   for (auto const &n:mapData.nodes){
-    projection.GeoToPixel(n->GetCoords(),x,y);
-    if (filterRectangle.contains(x,y)){
+    osmscout::Vertex2D screenPos;
+    projection.GeoToPixel(n->GetCoords(),
+                          screenPos);
+    if (filterRectangle.contains(screenPos.GetX(),screenPos.GetY())){
       addObjectInfo(objectList, n, reverseLookupMap, db, regionMap);
     }
   }
@@ -241,9 +241,14 @@ void LookupModule::filterObjectInView(const osmscout::MapData &mapData,
   for (auto const &w:mapData.ways){
     // TODO: better detection
     osmscout::GeoBox bbox=w->GetBoundingBox();
-    projection.GeoToPixel(bbox.GetMinCoord(),x,y);
-    projection.GeoToPixel(bbox.GetMaxCoord(),x2,y2);
-    if (filterRectangle.intersects(QRectF(QPointF(x,y),QPointF(x2,y2)))){
+    osmscout::Vertex2D minScreenPos;
+    osmscout::Vertex2D maxScreenPos;
+    projection.GeoToPixel(bbox.GetMinCoord(),
+                          minScreenPos);
+    projection.GeoToPixel(bbox.GetMaxCoord(),
+                          maxScreenPos);
+    if (filterRectangle.intersects(QRectF(QPointF(minScreenPos.GetX(),minScreenPos.GetY()),
+                                          QPointF(maxScreenPos.GetX(),maxScreenPos.GetY())))){
       addObjectInfo(objectList, w, reverseLookupMap, db, regionMap);
     }
   }
@@ -252,9 +257,14 @@ void LookupModule::filterObjectInView(const osmscout::MapData &mapData,
   for (auto const &a:mapData.areas){
     // TODO: better detection
     osmscout::GeoBox bbox=a->GetBoundingBox();
-    projection.GeoToPixel(bbox.GetMinCoord(),x,y);
-    projection.GeoToPixel(bbox.GetMaxCoord(),x2,y2);
-    if (filterRectangle.intersects(QRectF(QPointF(x,y),QPointF(x2,y2)))){
+    osmscout::Vertex2D minScreenPos;
+    osmscout::Vertex2D maxScreenPos;
+    projection.GeoToPixel(bbox.GetMinCoord(),
+                          minScreenPos);
+    projection.GeoToPixel(bbox.GetMaxCoord(),
+                          maxScreenPos);
+    if (filterRectangle.intersects(QRectF(QPointF(minScreenPos.GetX(),minScreenPos.GetY()),
+                                          QPointF(maxScreenPos.GetY(),maxScreenPos.GetY())))){
       addObjectInfo(objectList, a, reverseLookupMap, db, regionMap);
     }
   }
@@ -297,7 +307,7 @@ void LookupModule::requestLocationDescription(const osmscout::GeoCoord location)
           count++;
 
           auto place = description.GetAtAddressDescription()->GetPlace();
-          emit locationDescription(location, db->path, description,
+          emit locationDescription(location, QString::fromStdString(db->path), description,
                                    BuildAdminRegionList(db, place.GetAdminRegion(), regionMap));
         }
 
@@ -310,7 +320,7 @@ void LookupModule::requestLocationDescription(const osmscout::GeoCoord location)
           count++;
 
           auto place = description.GetAtPOIDescription()->GetPlace();
-          emit locationDescription(location, db->path, description,
+          emit locationDescription(location, QString::fromStdString(db->path), description,
                                    BuildAdminRegionList(db, place.GetAdminRegion(), regionMap));
         }
       }
@@ -349,19 +359,19 @@ AdminRegionInfoRef LookupModule::buildAdminRegionInfo(const DBInstanceRef &db,
     case osmscout::refNode:
       if (database->GetNodeByOffset(region->object.GetFileOffset(), node)) {
         adminLevelValue=adminLevelReader.GetValue(node->GetFeatureValueBuffer());
-        info->type=QString::fromStdString(node->GetType()->GetName());
+        info->type=node->GetType()->GetName();
       }
       break;
     case osmscout::refWay:
       if (database->GetWayByOffset(region->object.GetFileOffset(), way)) {
         adminLevelValue=adminLevelReader.GetValue(way->GetFeatureValueBuffer());
-        info->type=QString::fromStdString(way->GetType()->GetName());
+        info->type=way->GetType()->GetName();
       }
       break;
     case osmscout::refArea:
       if (database->GetAreaByOffset(region->object.GetFileOffset(), area)) {
         adminLevelValue=adminLevelReader.GetValue(area->GetFeatureValueBuffer());
-        info->type=QString::fromStdString(area->GetType()->GetName());
+        info->type=area->GetType()->GetName();
       }
       break;
     case osmscout::refNone:
@@ -391,7 +401,7 @@ void LookupModule::requestRegionLookup(const osmscout::GeoCoord location) {
         std::list<osmscout::LocationDescriptionService::ReverseLookupResult> result;
 
         if (db->GetLocationDescriptionService()->ReverseLookupRegion(location,result)){
-          std::map<osmscout::FileOffset,AdminRegionInfoRef> adminRegionMap=adminRegionCache[db->path];
+          std::map<osmscout::FileOffset,AdminRegionInfoRef> adminRegionMap=adminRegionCache[QString::fromStdString(db->path)];
           AdminRegionInfoRef bottomAdminRegion; // admin region with highest admin level
 
           for (const auto &entry:result) {
@@ -425,7 +435,7 @@ void LookupModule::requestRegionLookup(const osmscout::GeoCoord location) {
             emit locationAdminRegions(location,
                                       adminRegionList);
           }
-          adminRegionCache[db->path]=adminRegionMap;
+          adminRegionCache[QString::fromStdString(db->path)]=adminRegionMap;
         }
       }
     }
@@ -487,7 +497,7 @@ QStringList LookupModule::AdminRegionNames(const QList<AdminRegionInfoRef> &regi
   QString previousName;
   QString name;
   for (const auto &region: regionList) {
-    name = (useAltNames && !region->altName().empty()) ? region->qStringAltName() : region->qStringName();
+    name = QString::fromStdString( (useAltNames && !region->altName().empty()) ? region->altName() : region->name());
     if (previousName != name) {
       result << name;
       previousName = name;
@@ -505,7 +515,7 @@ QStringList LookupModule::IndexedAdminRegionNames(const QList<AdminRegionInfoRef
   }
   for (const auto &region:regionList) {
     if (region->adminLevel >= 0 && region->adminLevel < result.size()) {
-      result[region->adminLevel] = (useAltNames && !region->altName().empty()) ? region->qStringAltName() : region->qStringName();
+      result[region->adminLevel] = QString::fromStdString((useAltNames && !region->altName().empty()) ? region->altName() : region->name());
     }
   }
   return result;
