@@ -47,19 +47,15 @@ MapWidget::MapWidget(QQuickItem* parent)
 
     DBThreadRef dbThread = OSMScoutQt::GetInstance().GetDBThread();
 
-    connect(settings.get(), &Settings::MapDPIChange,
-            this, &MapWidget::onMapDPIChange);
+    settings->mapDPIChange.Connect(mapDpiSlot);
+    dbThread->stylesheetFilenameChanged.Connect(stylesheetFilenameChangedSlot);
+    dbThread->styleErrorsChanged.Connect(styleErrorsChangedSlot);
+    dbThread->databaseLoadFinished.Connect(databaseLoadedSlot);
 
     tapRecognizer.setPhysicalDpi(dbThread->GetPhysicalDpi());
 
     connect(renderer, &MapRenderer::Redraw,
             this, &MapWidget::redraw);
-    connect(dbThread.get(), &DBThread::stylesheetFilenameChanged,
-            this, &MapWidget::stylesheetFilenameChanged);
-    connect(dbThread.get(), &DBThread::styleErrorsChanged,
-            this, &MapWidget::styleErrorsChanged);
-    connect(dbThread.get(), &DBThread::databaseLoadFinished,
-            this, &MapWidget::databaseLoaded);
 
     connect(&tapRecognizer, &TapRecognizer::tap,        this, &MapWidget::onTap);
     connect(&tapRecognizer, &TapRecognizer::doubleTap,  this, &MapWidget::onDoubleTap);
@@ -309,9 +305,9 @@ void MapWidget::paint(QPainter *painter)
     // render vehicle
     if (vehicle.position && !vehicle.getIcon().isNull()){
       QImage vehicleIcon=vehicle.getIcon();
-      double x;
-      double y;
-      projection.GeoToPixel(vehicle.position->getCoord(), x, y);
+      osmscout::Vertex2D screenPos;
+      projection.GeoToPixel(vehicle.position->getCoord(),
+                            screenPos);
 
       Bearing iconAngle;
       if (vehicle.position->getBearing()) {
@@ -321,26 +317,32 @@ void MapWidget::paint(QPainter *painter)
       }
 
       painter->save();
-      painter->translate(x,y);
+      painter->translate(screenPos.GetX(),
+                         screenPos.GetY());
       painter->rotate(iconAngle.AsDegrees());
       // draw vehicleIcon center on coordinate 0x0
-      painter->drawImage(QPointF(vehicleIcon.width()/-2, vehicleIcon.height()/-2), vehicleIcon);
+      painter->drawImage(QPointF(vehicleIcon.width()/-2,
+                                 vehicleIcon.height()/-2),
+                         vehicleIcon);
       painter->restore();
     }
 
     // render current position spot
     if (showCurrentPosition && currentPosition.valid){
-        double x;
-        double y;
-        projection.GeoToPixel(currentPosition.coord, x, y);
-        if (boundingBox.contains(x, y)){
+        osmscout::Vertex2D screenPos;
+        projection.GeoToPixel(currentPosition.coord,
+                              screenPos);
+        if (boundingBox.contains(screenPos.GetX(), screenPos.GetY())){
 
             if (currentPosition.horizontalAccuracyValid){
                 double diameter = currentPosition.horizontalAccuracy * projection.GetMeterInPixel();
                 if (diameter > 25.0 && diameter < std::max(request.width, request.height)){
                     painter->setBrush(QBrush(QColor::fromRgbF(1.0, 1.0, 1.0, 0.4)));
                     painter->setPen(QColor::fromRgbF(1.0, 1.0, 1.0, 0.7));
-                    painter->drawEllipse(x - (diameter /2.0), y - (diameter /2.0), diameter, diameter);
+                    painter->drawEllipse(screenPos.GetX() - (diameter /2.0),
+                                         screenPos.GetY() - (diameter /2.0),
+                                         diameter,
+                                         diameter);
                 }
             }
 
@@ -353,14 +355,15 @@ void MapWidget::paint(QPainter *painter)
             }
             painter->setPen(QColor::fromRgbF(0.0, 0.5, 0.0, 0.9));
             double dimension = projection.ConvertWidthToPixel(2.8);
-            painter->drawEllipse(x - dimension/2, y - dimension/2, dimension, dimension);
+            painter->drawEllipse(screenPos.GetX() - dimension/2,
+                                 screenPos.GetY() - dimension/2,
+                                 dimension,
+                                 dimension);
         }
     }
 
     // render marks
     if (!marks.isEmpty()){
-        double x;
-        double y;
         painter->setBrush(QBrush());
         QPen pen;
         pen.setColor(QColor::fromRgbF(0.8, 0.0, 0.0, 0.9));
@@ -368,10 +371,14 @@ void MapWidget::paint(QPainter *painter)
         painter->setPen(pen);
 
         for (auto &entry: marks){
-            projection.GeoToPixel(osmscout::GeoCoord(entry.GetLat(), entry.GetLon()), x, y);
-            if (boundingBox.contains(x, y)){
+            osmscout::Vertex2D screenPos;
+            projection.GeoToPixel(osmscout::GeoCoord(entry.GetLat(), entry.GetLon()),
+                                  screenPos);
+            if (boundingBox.contains(screenPos.GetX(), screenPos.GetY())){
                 double dimension = projection.ConvertWidthToPixel(6);
-                painter->drawEllipse(x - dimension/2, y - dimension/2, dimension, dimension);
+                painter->drawEllipse(screenPos.GetX() - dimension/2,
+                                     screenPos.GetY() - dimension/2,
+                                     dimension, dimension);
             }
         }
     }
@@ -380,40 +387,40 @@ void MapWidget::paint(QPainter *painter)
 void MapWidget::recenter()
 {
   DBThreadRef dbThread = OSMScoutQt::GetInstance().GetDBThread();
-  DatabaseLoadedResponse resp = dbThread->loadedResponse();
-  if (!resp.boundingBox.IsValid()){
+  GeoBox boundingBox = dbThread->databaseBoundingBox();
+  if (!boundingBox.IsValid()){
     return;
   }
-  Distance dimension = osmscout::GetEllipsoidalDistance(resp.boundingBox.GetMinCoord(),
-                                                        resp.boundingBox.GetMaxCoord());
+  Distance dimension = osmscout::GetEllipsoidalDistance(boundingBox.GetMinCoord(),
+                                                        boundingBox.GetMaxCoord());
 
-  showCoordinates(resp.boundingBox.GetCenter(), magnificationByDimension(dimension));
+  showCoordinates(boundingBox.GetCenter(), magnificationByDimension(dimension));
 }
 
 bool MapWidget::isDatabaseLoaded()
 {
   DBThreadRef dbThread = OSMScoutQt::GetInstance().GetDBThread();
-  DatabaseLoadedResponse resp = dbThread->loadedResponse();
-  return resp.boundingBox.IsValid();
+  GeoBox boundingBox = dbThread->databaseBoundingBox();
+  return boundingBox.IsValid();
 }
 
 bool MapWidget::isInDatabaseBoundingBox(double lat, double lon)
 {
   DBThreadRef dbThread = OSMScoutQt::GetInstance().GetDBThread();
-  DatabaseLoadedResponse resp = dbThread->loadedResponse();
-  if (!resp.boundingBox.IsValid()){
+  GeoBox boundingBox = dbThread->databaseBoundingBox();
+  if (!boundingBox.IsValid()){
     return false;
   }
   osmscout::GeoCoord coord(lat, lon);
-  return resp.boundingBox.Includes(coord);
+  return boundingBox.Includes(coord);
 }
 
 QPointF MapWidget::screenPosition(double lat, double lon)
 {
-    double x;
-    double y;
-    getProjection().GeoToPixel(osmscout::GeoCoord(lat, lon), x, y);
-    return QPointF(x, y);
+    osmscout::Vertex2D screenPos;
+    getProjection().GeoToPixel(osmscout::GeoCoord(lat, lon),
+                               screenPos);
+    return QPointF(screenPos.GetX(), screenPos.GetY());
 }
 
 void MapWidget::zoom(double zoomFactor)
@@ -543,6 +550,9 @@ void MapWidget::setLockToPosition(bool lock){
 }
 
 void MapWidget::setFollowVehicle(bool follow){
+  if (vehicle.follow == follow) {
+    return;
+  }
   vehicle.follow = follow;
   vehicle.lastGesture.invalidate(); // set to invalid state
   if (follow){
@@ -766,7 +776,8 @@ void MapWidget::onIconFound(QPoint /*lookupCoord*/, MapIcon icon)
 
   emit iconTapped(icon.screenCoord, icon.coord.GetLat(), icon.coord.GetLon(), icon.databasePath,
                   QString(icon.objectRef.GetTypeName()), icon.objectRef.GetFileOffset(), icon.poiId,
-                  icon.type, icon.name, icon.altName, icon.ref, icon.operatorName, icon.phone, icon.website);
+                  icon.type, icon.name, icon.altName, icon.ref, icon.operatorName, icon.phone, icon.website,
+                  icon.openingHours);
 }
 
 void MapWidget::onDoubleTap(const QPoint p)
@@ -806,6 +817,8 @@ void MapWidget::onTapLongTap(const QPoint p)
 
 void MapWidget::onMapDPIChange(double dpi)
 {
+    assert(QThread::currentThread() == this->thread());
+
     MapView v = *view;
     v.mapDpi = dpi;
     changeView(v);
@@ -914,7 +927,7 @@ void MapWidget::setInteractiveIcons(bool b)
 QString MapWidget::GetStylesheetFilename() const
 {
     DBThreadRef dbThread=OSMScoutQt::GetInstance().GetDBThread();
-    return dbThread->GetStylesheetFilename();
+    return QString::fromStdString(dbThread->GetStylesheetFilename());
 }
 
 QString MapWidget::GetZoomLevelName() const
@@ -924,7 +937,7 @@ QString MapWidget::GetZoomLevelName() const
 
     if (converter.Convert(osmscout::MagnificationLevel(view->magnification.GetLevel()),
                           name)) {
-      return name.c_str();
+      return QString::fromStdString(name);
     }
 
     return "";
@@ -932,31 +945,34 @@ QString MapWidget::GetZoomLevelName() const
 bool MapWidget::stylesheetHasErrors() const
 {
     DBThreadRef dbThread=OSMScoutQt::GetInstance().GetDBThread();
-    return !dbThread->GetStyleErrors().isEmpty();
+    return !dbThread->GetStyleErrors().empty();
 }
 int MapWidget::firstStylesheetErrorLine() const
 {
     DBThreadRef dbThread=OSMScoutQt::GetInstance().GetDBThread();
-    QList<StyleError> errors=dbThread->GetStyleErrors();
-    if (errors.isEmpty())
+    std::list<StyleError> errors=dbThread->GetStyleErrors();
+    if (errors.empty()) {
       return -1;
-    return errors.first().GetLine();
+    }
+    return errors.front().GetLine();
 }
 int MapWidget::firstStylesheetErrorColumn() const
 {
     DBThreadRef dbThread=OSMScoutQt::GetInstance().GetDBThread();
-    QList<StyleError> errors=dbThread->GetStyleErrors();
-    if (errors.isEmpty())
+    std::list<StyleError> errors=dbThread->GetStyleErrors();
+    if (errors.empty()) {
       return -1;
-    return errors.first().GetColumn();
+    }
+    return errors.front().GetColumn();
 }
 QString MapWidget::firstStylesheetErrorDescription() const
 {
     DBThreadRef dbThread=OSMScoutQt::GetInstance().GetDBThread();
-    QList<StyleError> errors=dbThread->GetStyleErrors();
-    if (errors.isEmpty())
+    std::list<StyleError> errors=dbThread->GetStyleErrors();
+    if (errors.empty()) {
       return "";
-    return errors.first().GetDescription();
+    }
+    return QString::fromStdString(errors.front().GetDescription());
 }
 
 bool MapWidget::toggleDebug()
