@@ -29,7 +29,7 @@
 #include <osmscout/system/Assert.h>
 
 #include <osmscout/util/Geometry.h>
-#include <osmscout/util/Logger.h>
+#include <osmscout/log/Logger.h>
 #include <osmscout/util/StopClock.h>
 
 #include <iomanip>
@@ -90,41 +90,20 @@ namespace osmscout {
   }
 
   template <class RoutingState>
-  void AbstractRoutingService<RoutingState>::ResolveRNodeChainToList(DBId finalRouteNode,
+  void AbstractRoutingService<RoutingState>::ResolveRNodeChainToList(const RNode &finalRouteNode,
                                                                      const ClosedSet& closedSet,
-                                                                     const ClosedSet& closedRestrictedSet,
                                                                      std::list<VNode>& nodes)
   {
-    bool restricted=false;
-    auto current=closedSet.find(VNode(finalRouteNode));
-
-    if (current==closedSet.end()){
-      current=closedRestrictedSet.find(VNode(finalRouteNode));
-      assert(current!=closedSet.end());
-      restricted=true;
-    }
+    auto current=closedSet.find(VNode(finalRouteNode.id, finalRouteNode.restricted));
+    assert(current!=closedSet.end());
 
     while (current->previousNode.IsValid()) {
       if constexpr (debugRouting) {
         std::cout << "Chain item " << current->currentNode << " -> " << current->previousNode << std::endl;
       }
       ClosedSet::const_iterator prev;
-      if (!restricted){
-        prev=closedSet.find(VNode(current->previousNode));
-        if (prev==closedSet.end()){
-          prev=closedRestrictedSet.find(VNode(current->previousNode));
-          assert(prev!=closedRestrictedSet.end());
-          restricted=true;
-        }
-      }else{
-        prev=closedRestrictedSet.find(VNode(current->previousNode));
-        if (prev==closedRestrictedSet.end()){
-          prev=closedSet.find(VNode(current->previousNode));
-          assert(prev!=closedSet.end());
-          restricted=false;
-        }
-      }
-
+      prev=closedSet.find(VNode(current->previousNode, current->previousRestricted));
+      assert(prev!=closedSet.end());
 
       nodes.push_back(*current);
 
@@ -383,6 +362,9 @@ namespace osmscout {
                   forwardRNode)) {
       return false;
     }
+    if (forwardRNode) {
+      forwardRNode->leaveRestricted=true;
+    }
 
     if (backwardRouteNode &&
         !GetRNode(state,
@@ -395,6 +377,9 @@ namespace osmscout {
                   backwardRNode)) {
       return false;
     }
+    if (backwardRNode) {
+      backwardRNode->leaveRestricted=true;
+    }
 
     return true;
   }
@@ -564,18 +549,14 @@ namespace osmscout {
                                                                   RouteNodeRef &currentRouteNode,
                                                                   OpenList &openList,
                                                                   OpenMap &openMap,
-                                                                  const ClosedSet &closedSet,
-                                                                  const ClosedSet &closedRestrictedSet)
+                                                                  const ClosedSet &closedSet)
   {
     // add twin nodes to nextNode from other databases to open list
     std::vector<DBId> twins=GetNodeTwins(state,
                                          current->id.database,
                                          currentRouteNode->GetId());
     for (const auto& twin : twins) {
-      if ((current->access &&
-           closedSet.find(VNode(twin))!=closedSet.end()) ||
-          (!current->access &&
-            closedRestrictedSet.find(VNode(twin))!=closedRestrictedSet.end())){
+      if (closedSet.find(VNode(twin, current->restricted))!=closedSet.end()) {
         if constexpr (debugRouting) {
           std::cout << "Twin node " << twin << " is closed already, ignore it" << std::endl;
         }
@@ -590,12 +571,13 @@ namespace osmscout {
           // this is cheaper path to twin
 
           rn->prev=current->id;
+          rn->prevRestricted=current->restricted;
           //rn->object=node->objects.begin()->object, /*TODO: how to find correct way from other DB?*/
 
           rn->currentCost=current->currentCost;
           rn->estimateCost=current->estimateCost;
           rn->overallCost=current->overallCost;
-          rn->access=current->access;
+          rn->restricted=current->restricted;
 
           openList.erase(twinIt->second);
 
@@ -616,12 +598,13 @@ namespace osmscout {
                                             node,
                                             //node->objects.begin()->object, /*TODO: how to find correct way from other DB?*/
                                             ObjectFileRef(), // TODO: have to be valid Object here?
-                                            /*prev*/current->id);
+                                            /*prev*/current->id,
+                                            current->restricted);
 
         rn->currentCost=current->currentCost;
         rn->estimateCost=current->estimateCost;
         rn->overallCost=current->overallCost;
-        rn->access=current->access;
+        rn->restricted=current->restricted;
 
         std::pair<OpenListRef,bool> insertResult=openList.insert(rn);
         openMap[rn->id]=insertResult.first;
@@ -641,7 +624,6 @@ namespace osmscout {
                                                        OpenList &openList,
                                                        OpenMap &openMap,
                                                        ClosedSet &closedSet,
-                                                       ClosedSet &closedRestrictedSet,
                                                        RoutingResult &result,
                                                        const RoutingParameter& parameter,
                                                        const GeoCoord &targetCoord,
@@ -666,7 +648,7 @@ namespace osmscout {
         }
         inPathIndex++;
       }
-      // in case of roundabout, node don't contains path back
+      // in case of roundabout, node don't contain path back
       inPathValid=inPathIndex < currentRouteNode->paths.size();
     }
 
@@ -686,8 +668,23 @@ namespace osmscout {
         continue;
       }
 
-      if (!current->access &&
-          !path.IsRestricted(vehicle)) {
+      if (path.id==current->exclude) {
+        if constexpr (debugRouting) {
+          std::cout << "  Skipping route";
+          std::cout << " to " << path.id;
+          std::cout << " (" << currentRouteNode->objects[path.objectIndex].object.GetTypeName() << " "
+                    << currentRouteNode->objects[path.objectIndex].object.GetFileOffset() << ")";
+          std::cout << " => special exclusion because of vehicle bearing" << std::endl;
+        }
+        nodesIgnoredCount++;
+        i++;
+
+        continue;
+      }
+
+      if (current->restricted &&
+          !path.IsRestricted(vehicle) &&
+          !current->leaveRestricted) {
         if constexpr (debugRouting) {
           std::cout << "  Skipping route";
           std::cout << " to " << path.id;
@@ -717,10 +714,7 @@ namespace osmscout {
         continue;
       }
 
-      if ((current->access &&
-           closedSet.find(VNode(DBId(dbId,path.id)))!=closedSet.end()) ||
-          (!current->access &&
-           closedRestrictedSet.find(VNode(DBId(dbId,path.id)))!=closedRestrictedSet.end())) {
+      if (closedSet.find(VNode(DBId(dbId,path.id), path.IsRestricted(vehicle)))!=closedSet.end()) {
         if constexpr (debugRouting) {
           std::cout << "  Skipping route";
           std::cout << " to " << dbId << " / " << path.id;
@@ -831,12 +825,18 @@ namespace osmscout {
         RNodeRef node=*openEntry->second;
 
         node->prev=current->id;
+        node->prevRestricted=current->restricted;
         node->object=currentRouteNode->objects[path.objectIndex].object;
 
         node->currentCost=currentCost;
         node->estimateCost=estimateCost;
         node->overallCost=overallCost;
-        node->access=!currentRouteNode->paths[i].IsRestricted(vehicle);
+        node->restricted=currentRouteNode->paths[i].IsRestricted(vehicle);
+        if (node->restricted &&
+            current->leaveRestricted) {
+          // allow to leave restricted area
+          node->leaveRestricted=true;
+        }
 
         if constexpr (debugRouting) {
           std::cout << "  Updating route " << current->id << " via " << node->object.GetTypeName() << " "
@@ -853,12 +853,18 @@ namespace osmscout {
         RNodeRef node=std::make_shared<RNode>(DBId(dbId,path.id),
                                               nextNode,
                                               currentRouteNode->objects[path.objectIndex].object,
-                                              current->id);
+                                              current->id,
+                                              current->restricted);
 
         node->currentCost=currentCost;
         node->estimateCost=estimateCost;
         node->overallCost=overallCost;
-        node->access=!path.IsRestricted(vehicle);
+        node->restricted=path.IsRestricted(vehicle);
+        if (node->restricted &&
+            current->leaveRestricted) {
+          // allow to leave restricted area
+          node->leaveRestricted=true;
+        }
 
         if constexpr (debugRouting) {
           std::cout << "  Inserting route to " << path.id;
@@ -877,6 +883,93 @@ namespace osmscout {
     return true;
   }
 
+  template <class RoutingState>
+  bool AbstractRoutingService<RoutingState>::RestrictInitialUTurn(const RoutingState& state,
+                                                                  const Bearing& bearing,
+                                                                  const RoutePosition& start,
+                                                                  RNodeRef startForwardNode,
+                                                                  RNodeRef startBackwardNode)
+  {
+    if (startForwardNode && startBackwardNode && start.GetObjectFileRef().GetType()==refWay) {
+      // start is on the way between two route nodes, lets evaluate what route node is in opposite
+      // direction and add u-turn penalty to it and avoid return from node ahead
+      WayRef way;
+      if (!GetWayByOffset(DBFileOffset(start.GetDatabaseId(),
+                                       start.GetObjectFileRef().GetFileOffset()),
+                          way)) {
+        log.Error() << "Cannot get start way!";
+        return false;
+      }
+      if (CanUseForward(state,start.GetDatabaseId(),way) &&
+          CanUseBackward(state,start.GetDatabaseId(),way) &&
+          start.GetNodeIndex() > 0 &&
+          start.GetNodeIndex() < way->nodes.size() -1) {
+        auto pos = way->nodes[start.GetNodeIndex()].GetCoord();
+        auto forwardPos = way->nodes[start.GetNodeIndex()+1].GetCoord();
+        auto backwardPos = way->nodes[start.GetNodeIndex()-1].GetCoord();
+        auto forwardBearing = GetSphericalBearingInitial(pos, forwardPos);
+        auto backwardBearing = GetSphericalBearingInitial(pos, backwardPos);
+        auto forwardBearingDiff = (forwardBearing-bearing).AsDegrees();
+        forwardBearingDiff = forwardBearingDiff > 180 ? 360 - forwardBearingDiff : forwardBearingDiff;
+        auto backwardBearingDiff = (backwardBearing-bearing).AsDegrees();
+        backwardBearingDiff = backwardBearingDiff > 180 ? 360 - backwardBearingDiff : backwardBearingDiff;
+
+        auto penalizedRNode = ( forwardBearingDiff < backwardBearingDiff ) ? startBackwardNode : startForwardNode;
+        if constexpr (debugRouting) {
+          std::cout << "Adding u-turn penalty to route node " << penalizedRNode->id;
+        }
+        auto aheadRNode = ( forwardBearingDiff < backwardBearingDiff ) ? startForwardNode : startBackwardNode;
+        penalizedRNode->currentCost += GetUTurnCost(state, penalizedRNode->id.database);
+        penalizedRNode->overallCost = penalizedRNode->currentCost + penalizedRNode->estimateCost;
+
+        // avoid return from aheadRNode to penalizedRNode
+        aheadRNode->exclude = penalizedRNode->id.id;
+      }
+    }
+    if (startForwardNode && !startBackwardNode && start.GetObjectFileRef().GetType()==refWay) {
+      // start is directly on route-node, try to find what path is our source and add exclusion to it
+      double restrictedPathBearing = 20; // degrees
+      auto oppositeVehicleBearing = bearing - Bearing::Degrees(180);
+      for (const auto& path : startForwardNode->node->paths) {
+        auto object = startForwardNode->node->objects[path.objectIndex].object;
+        if (object.GetType() != refWay) {
+          continue;
+        }
+        WayRef way;
+        if (!GetWayByOffset(DBFileOffset(startForwardNode->id.database,
+                                         object.GetFileOffset()),
+                            way)) {
+          log.Error() << "Cannot get starting junction way!";
+          return false;
+        }
+
+        int fromNode=-1;
+        int toNode=-1;
+        for (size_t i= 0; i < way->nodes.size() && (fromNode<0 || toNode<0); i++) {
+          if (way->nodes[i].GetId() == startForwardNode->node->GetId()) {
+            fromNode = i;
+          }
+          if (way->nodes[i].GetId() == path.id) {
+            toNode = i;
+          }
+        }
+        assert(fromNode>=0);
+        assert(toNode>=0);
+        assert(fromNode!=toNode);
+        auto pathBearing = fromNode < toNode ?
+                           GetSphericalBearingInitial(way->nodes[fromNode].GetCoord(), way->nodes[fromNode+1].GetCoord()) :
+                           GetSphericalBearingInitial(way->nodes[fromNode].GetCoord(), way->nodes[fromNode-1].GetCoord());
+        auto bearingDiff = (pathBearing - oppositeVehicleBearing).AsDegrees();
+        bearingDiff = bearingDiff > 180 ? 360 - bearingDiff : bearingDiff;
+        if (bearingDiff < restrictedPathBearing) {
+          startForwardNode->exclude = path.id;
+          restrictedPathBearing = bearingDiff;
+        }
+      }
+    }
+    return true;
+  }
+
   /**
    * Calculate a route
    *
@@ -886,6 +979,8 @@ namespace osmscout {
    *    Start of the route
    * @param target
    *    Target of teh route
+   * @param bearing
+   *    Initial vehicle bearing, route will start by the way in specified direction. When possible.
    * @param progress
    *    Optional callback for handling routing progress
    * @param route
@@ -897,6 +992,7 @@ namespace osmscout {
   RoutingResult AbstractRoutingService<RoutingState>::CalculateRoute(RoutingState& state,
                                                                      const RoutePosition& start,
                                                                      const RoutePosition& target,
+                                                                     const std::optional<osmscout::Bearing> &bearing,
                                                                      const RoutingParameter& parameter)
   {
     RoutingResult            result;
@@ -917,12 +1013,7 @@ namespace osmscout {
     // Map routing nodes by id
     OpenMap                  openMap;
 
-    // Restricted way (access=destination) is a way that may be used just
-    // in case when target is on this way. Some routing nodes may be accessed
-    // from two different ways - one without any access restriction (closedSet)
-    // and second with restriction (closedRestrictedSet)
     ClosedSet                closedSet;
-    ClosedSet                closedRestrictedSet;
 
     size_t                   nodesLoadedCount=0;
     size_t                   nodesIgnoredCount=0;
@@ -931,7 +1022,6 @@ namespace osmscout {
 
     openMap.reserve(10000);
     closedSet.reserve(300000);
-    closedRestrictedSet.reserve(10000);
 
     if (!GetTargetNodes(state,
                         target,
@@ -976,6 +1066,12 @@ namespace osmscout {
       return result;
     }
 
+    if (bearing) {
+      if (!RestrictInitialUTurn(state, *bearing, start, startForwardNode, startBackwardNode)) {
+        return result;
+      }
+    }
+
     if (startForwardNode) {
       std::pair<OpenListRef,bool> insertResult=openList.insert(startForwardNode);
 
@@ -1041,7 +1137,6 @@ namespace osmscout {
                      openList,
                      openMap,
                      closedSet,
-                     closedRestrictedSet,
                      result,
                      parameter,
                      targetCoord,
@@ -1065,8 +1160,7 @@ namespace osmscout {
                                 currentRouteNode,
                                 openList,
                                 openMap,
-                                closedSet,
-                                closedRestrictedSet)) {
+                                closedSet)) {
         log.Error() << "Failed to walk to other databases from " << dbId << " / " << currentRouteNode->GetFileOffset();
         return result;
       }
@@ -1078,21 +1172,16 @@ namespace osmscout {
       if constexpr (debugRouting) {
         std::cout << "Closing " << current->id << " (previous " << current->prev << ")" << std::endl;
       }
-      if (current->access) {
-        closedSet.insert(VNode(current->id,
-                               current->object,
-                               current->prev));
-      }
-      else {
-        closedRestrictedSet.insert(VNode(current->id,
-                                         current->object,
-                                         current->prev));
-      }
+      closedSet.insert(VNode(current->id,
+                             current->restricted,
+                             current->object,
+                             current->prev,
+                             current->prevRestricted));
 
       current->node=nullptr;
 
       maxOpenList=std::max(maxOpenList,openMap.size());
-      maxClosedSet=std::max(maxClosedSet,closedSet.size()+closedRestrictedSet.size());
+      maxClosedSet=std::max(maxClosedSet,closedSet.size());
 
       if constexpr (debugRouting) {
         if (openList.empty()) {
@@ -1132,10 +1221,12 @@ namespace osmscout {
 
     // If we have keep the last node open because of access violations, add it
     // after routing is done
-    if (closedSet.find(VNode(current->id))==closedSet.end()) {
+    if (closedSet.find(VNode(current->id, current->restricted))==closedSet.end()) {
       closedSet.insert(VNode(current->id,
+                             current->restricted,
                              current->object,
-                             current->prev));
+                             current->prev,
+                             current->prevRestricted));
     }
     RNodeRef  targetFinalNode;
 
@@ -1222,9 +1313,8 @@ namespace osmscout {
       return result;
     }
 
-    ResolveRNodeChainToList(targetFinalNode->id,
+    ResolveRNodeChainToList(*targetFinalNode,
                             closedSet,
-                            closedRestrictedSet,
                             nodes);
 
     if constexpr (debugRouting) {
@@ -1589,7 +1679,7 @@ namespace osmscout {
 
       if (n->currentNode.database!=nn->currentNode.database &&
           node->GetId()==nextNode->GetId()){
-        // there is no way between database transition nodes
+        // there is no way between db transition nodes
         continue;
       }
 
